import { Prop, Schema, SchemaFactory } from '@nestjs/mongoose';
import { HydratedDocument } from 'mongoose';
import { Decision } from './Decision.schema';
import { ApplicationStatus, ObjectStatus } from 'src/enums/status.enum';
import { ICAInformation } from './ICAInformation.schema';
import { Exclude, Expose, Transform } from 'class-transformer';

export type IndustryChangeApplicationDocument =
  HydratedDocument<IndustryChangeApplication>;

@Schema({
  timestamps: true,
})
export class IndustryChangeApplication {
  @Expose({ name: 'id' })
  @Transform(({ key, obj }) => obj[key].toString())
  _id: string;

  @Exclude()
  __v: number;

  @Prop({
    required: true,
  })
  residentSub: string;

  @Prop({
    required: true,
    type: ICAInformation,
  })
  current: ICAInformation;

  @Prop({
    required: true,
    type: ICAInformation,
  })
  requested: ICAInformation;

  @Prop({
    type: String,
    enum: ApplicationStatus,
    default: ApplicationStatus.IN_REVIEW,
    required: true,
  })
  status: ApplicationStatus;

  @Prop()
  submittedAt: Date;

  @Prop({ type: Decision })
  decision: Decision;

  @Exclude()
  @Prop()
  createdBy: string;

  @Exclude()
  @Prop()
  updatedBy: string;

  @Prop({
    type: String,
    enum: ObjectStatus,
    default: ObjectStatus.CURRENT,
    required: true,
  })
  objectStatus: ObjectStatus;

  @Exclude()
  createdAt: Date;

  @Exclude()
  updatedAt: Date;
}

export const IndustryChangeApplicationSchema = SchemaFactory.createForClass(
  IndustryChangeApplication,
);
